import 'package:eventizer/EventID.dart';

/// This class was created to integrate event id and data returned into one.
/// this will help in handling of data and make is compact of every other class
class Event {
  /// Event id for identification
  final EventID id;

  /// Event data that is required for proper handling of this event
  final dynamic data;

  /// Construct Event Data object withh passed [id] and optional [data] with a Typed type
  Event(this.id, [data]) : this.data = data;

  T get<T>({dynamic id = 0}) {
    assert((data?.runtimeType ?? T) == T,
        'type mismatch expected $T found ${data?.runtimeType}');
    return data;
  }
}

class EventMulti extends Event {
  EventMulti(id, List<dynamic> datal) : super(id) {
    datal.forEach(
      (e) => data[e.runtimeType] = (data[e.runtimeType] ?? []).add(e),
    );
  }

  @override
  T get<T>({dynamic id = 0}) {
    assert(data[T].length <= id, 'index out of boud');
    assert(data[T][id]?.runtimeType == T,
        'type mismatch expected $T found ${data[T][id]?.runtimeType}');
    return data[T][id];
  }
}

class EventMap extends Event {
  EventMap(id, Map<String, dynamic> data) : super(id, data);

  @override
  T get<T>({dynamic id = 0}) {
    assert(data[T].length <= id, 'index out of boud');
    assert(data[T][id]?.runtimeType == T,
        'type mismatch expected $T found ${data[T][id]?.runtimeType}');
    return data[T][id];
  }
}
