import 'package:eventizer/EventID.dart';
import 'package:eventizer/EventDebug.dart';
import 'package:eventizer/EventData.dart';

/// final but private [Map] of [List] of [EventConsumer]
/// this structure holds all active listener for a perticular [EventID]
/// this structure if used whenever a event with EventID is triggered
/// TODO make it thread safe
final Map<EventID, List<EventConsumer>> _consumerList = {};

/// abstarct event consumer class because each event is unique in its own way,
/// its handling is also unique thus this class is abstract
/// user must extend, mixin with this class to use the power of eventizer consumer interface
/// optionally user can also set debug tag
abstract class EventConsumer {
  ///
  String _debugTag = 'EventConsumer';

  /// set debug tag to [dt]
  set debugTag(dt) => _debugTag = dt;

  /// adds this EventConsumer entry to the global list for the provided list of EventID [list]
  void addEventHandler(List<EventID> list) {
    eventLog(LOG_TYPE_INFO, 'eventizer', 'Registering $_debugTag for $list');
    list.forEach((id) {
      _consumerList[id] ??= [];
      if (!_consumerList[id].contains(this)) {
        _consumerList[id].add(this);
      }
    });
  }

  /// removes this EventConsumer entry from the global list for the provided list of EventID [list]
  void removeEventHandler(List<EventID> list) {
    eventLog(LOG_TYPE_INFO, 'eventizer', 'Removing $_debugTag for $list');
    list.forEach((id) {
      _consumerList[id]?.remove(this);
    });
  }

  ///
  void event(Event event);
}

/// if a class is expected to generate event (mainly ui or async fucntion).
/// then they must extend, mixin with this class to trigger events
class EventProducer {
  /// trigger this [event] and notifies all registered consumer with optional [args]
  void triggerEvent(Event event) {
    eventLog(LOG_TYPE_INFO, 'eventizer',
        'Event Fired ${event.id} with ${event.data}');
    _consumerList[event.id]?.forEach((handler) => handler.event(event));
  }
}
